﻿define( ['qvangular',
		'text!QvServiceNowConnector.webroot/connectdialog.ng.html',
        'css!QvServiceNowConnector.webroot/connectdialog.css'
], function (qvangular, template) {
	return {
		template: template,
        controller: ['$scope', 'input', function ($scope, input) {

            function getParameter(parameterName, scopeMember) {
                return input.serverside.sendJsonRequest("getConnectionParameter", parameterName).then(function (info) {
                    console.log("info", info, scopeMember)
                    $scope[scopeMember] = info.qMessage;
                    return qvangular.promise(info.qMessage);
                });
            }

			function init() {
                //gui Seyup
                $scope.isEdit = input.editMode;
				$scope.id = input.instanceId;
				$scope.titleText = $scope.isEdit ? "Change ServiceNow Connector" : "Add ServiceNow Connector";
				$scope.saveButtonText = $scope.isEdit ? "Save changes" : "Create";

                //GUI Variables
                $scope.serviceNowEndpoint = "";                
                $scope.userName = "";
                $scope.password = "";

                //Qlik Variables
                $scope.name = "";
                $scope.provider = "ServiceNowConnector.exe"; // Connector filename

                //gui variables
                $scope.connectionInfo = "";
                $scope.connectionSuccessful = false;
                $scope.testingInProgress = false;

                //initial connection string
                $scope.connectionString = createCustomConnectionString($scope.provider, $scope.serviceNowEndpoint);



                //Get info fom Connector for Debug
				input.serverside.sendJsonRequest( "getInfo" ).then( function ( info ) {
					$scope.info = info.qMessage;
				} );

                if ($scope.isEdit) {

                    //get connection name
                    input.serverside.getConnection($scope.id).then(function (result) {
                        console.log("result", result);
                        $scope.name = result.qConnection.qName;
                    });

                    //get custom parameters
                    getParameter("serviceNowEndpoint", "serviceNowEndpoint");
                    getParameter("UserId", "userName").then(function(name){
                        $scope.currentServiceAccountName = name;
                    });
                    
                }

            }

            console.log('org connect dialog', Date.now(), input);

            //functions

            function validScopeInput(scopeVariableArray) {
                for (index = 0; index < scopeVariableArray.length; ++index) {
                    scopeVariable = scopeVariableArray[index];
                    //console.log("test", scopeVariable.property, $scope[scopeVariable.property], $scope[scopeVariable.property].length, !!$scope[scopeVariable.property], $scope[scopeVariable.property].length > 0, !!scopeVariable.regex, scopeVariable.regex);
                    if (!!$scope[scopeVariable.property] && $scope[scopeVariable.property].length > 0) {
                        //has regex?
                        if (!!scopeVariable.regex) {                            
                            if (!scopeVariable.regex.test($scope[scopeVariable.property])) {
                                return { message: "invalid format " + scopeVariable.regexError + "", valid: false };
                            } 
                        }
                        //ok
                    } else {
                        return { message: "missing " + scopeVariable.error, valid: false };
                    }
                }
                return { message: "Input ok", valid: true };
            }


			/* Event handlers */

			$scope.onOKClicked = function () {
				if ( $scope.isEdit ) {
                    var overrideCredentials = ($scope.userName !== "" && $scope.password !== "");
                    $scope.connectionString = createCustomConnectionString($scope.provider, $scope.serviceNowEndpoint);
                    console.log("updating connection with name", $scope.name, "username", $scope.userName, "connection string", $scope.connectionString);
                    input.serverside.modifyConnection( $scope.id,
						$scope.name,
						$scope.connectionString,
						$scope.provider,
						overrideCredentials,
						$scope.userName,
						$scope.password ).then( function ( result ) {
                            if (result) {
                                //console.log("result", result);
								$scope.destroyComponent();
							}
						} );
				} else {
                    $scope.connectionString = createCustomConnectionString($scope.provider, $scope.serviceNowEndpoint );
                    input.serverside.createNewConnection($scope.name, $scope.connectionString, $scope.userName, $scope.password);
					$scope.destroyComponent();
				}
			};

            $scope.onTestConnectionClicked = function () {

                $scope.connectionInfo = "";

                var scopeVariables = [
                    { property: "name", error: "connection name" },
                    { property: "userName", error: "connector name", regex: /[a-zA-Z0-9_]+$/, regexError: "only a-z A-Z 0-9 and _ is allowed" },
                    { property: "serviceNowEndpoint", error: "serviceNowEndpoint" },
                ];

                if (!$scope.isEdit || $scope.userName != $scope.currentServiceAccountName ) {
                    scopeVariables.push({ property: "password", error: "json key" });
                }

                var inputValidation = validScopeInput(scopeVariables);

                if (!inputValidation.valid) {
                    $scope.connectionInfo = inputValidation.message;
                    $scope.connectionSuccessful = false;
                    return;
                }


                $scope.testingInProgress = true;
                input.serverside.sendJsonRequest("testConnection", $scope.userName, $scope.password, $scope.serviceNowEndpoint).then(function (info) {
                    console.log("Info", info);
                    $scope.connectionInfo = info.message;
                    $scope.connectionSuccessful = info.access;
                    $scope.testingInProgress = false;
                    console.log($scope.connectionSuccessful);
				} );
			};

            $scope.isOkEnabled = function () {
                return !!$scope.name && $scope.name.length > 0 && $scope.connectionSuccessful && !$scope.testingInProgress;
			};

			$scope.onEscape = $scope.onCancelClicked = function () {
				$scope.destroyComponent();
            };

            /* Watches */
			
			/* Helper functions */


            function createCustomConnectionString(provider, serviceNowEndpoint) {

                var connectionString = "CUSTOM CONNECT TO " + "\"provider=" + provider + ";serviceNowEndpoint=" + serviceNowEndpoint + ";";

                console.log("string", connectionString);
                return connectionString;
            }

            // Help Menu


			init();
		}]
	};
} );