define(['qvangular',
    'underscore',
    'text!QvServiceNowConnector.webroot/selectdialog.ng.html',
    'css!QvServiceNowConnector.webroot/selectdialogstyle.css'
], function (qvangular, _, template) {
    return {
        template: template,
        controller: ['$scope', 'serverside', function ($scope, serverside) {
            var serviceNowDialogContentProvider = {
                getConnectionInfo: function () {
                    return qvangular.promise({
                        dbusage: false,
                        ownerusage: false,
                        dbseparator: '.',
                        ownerseparator: '.',
                        specialchars: '! "$&\'()*+,-/:;<>`{}~[]',
                        quotesuffix: '"',
                        quoteprefix: '"',
                        dbfirst: true,
                        keywords: []
                    });
                },



                getPreview: function ( /*databaseName, ownerName, tableName*/) {
                    return qvangular.promise([]);
                },

                
                getTables: function () {
                    
                    //return qvangular.promise([
                    //    {
                    //        name: projectID + " " + dataSetID + " first table",
                    //        id: "T01"
                    //    },
                    //    {
                    //        name: projectID + " " + dataSetID + " second table",
                    //        id: "T02"
                    //    },
                    //]);

                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getTables").then(function (response) {
                        $scope.loadInProgress = false;

                        console.log("tables response", response)
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No tables was returned";
                            console.log($scope.errorMessage);
                            return [{ "qName": "No tables was returned" }];
                        }



                        return qvangular.promise(response.Tables);
                    });
                },
                getFields: function (tableID) {
                    console.log("Field - Table", tableID );

                    //return qvangular.promise([
                    //    {
                    //        name: projectID + " " + dataSetID + " " + tableID + " first field",
                    //        id: "F01"
                    //    },
                    //    {
                    //        name: projectID + " " + dataSetID + " " + tableID + " second field",
                    //        id: "F02"
                    //    },
                    //]);

                    $scope.loadInProgress = true;

                    return serverside.sendJsonRequest("getFields", tableID).then(function (response) {
                        $scope.loadInProgress = false;

                        console.log("Field response", response)
                        if (response.type && response.type == "Error") {
                            $scope.errorMessage = "No fields was returned";
                            console.log($scope.errorMessage);
                            return [{ "qName": "No fields was returned" }];
                        }
                        return qvangular.promise(response.Fields);
                    });
                },

            };


            //helpers
            var DEBUG = true;

            //wrapping the logging function
            var log = function () {
                if (DEBUG) {
                    console.log.apply(console, arguments);
                }
            }

            //Filters

            //init of scope
            function init() {

                //Model variables 
                $scope.tables = [];
                $scope.fields = [];


                $scope.errorMessage = "";
                $scope.loadInProgress = false;
                $scope.serviceNowQuery = "";

                $scope.selectDialogState = "OK";

                //View control variables
                $scope.dataview = false;
                $scope.helpview = false;
                $scope.metadataview = true;

                //GUI mode constrol
                $scope.showSelectionGUI = true;

                //get datasets in project
                serviceNowDialogContentProvider.getTables().then(function (tables) {
                    console.log("tables", tables);
                    $scope.tables = tables;
                });                
            
            }


            console.log('org select dialog', Date.now(), $scope, serviceNowDialogContentProvider);


            //GUI fucntions

            function getFields(tableID) {
                //get table
                var table = $scope.tables.find(table => table.name === tableID);
                console.log("getFields", table, !!table.fields );
                if (!table.fields) {
                    //get fields
                    //$scope.getFields($scope.currentDataSet.id, table.id);

                    return serviceNowDialogContentProvider.getFields(tableID).then(function (fields) {
                        console.log("got Fields", fields);

                        //find table in scope and add fields

                        table.fields = fields; //add fields to table
                        table.numberOfSelectedFields = 0; //set number of selected
                        //$scope.fields = table.fields; //assign fieds to current fields

                        return qvangular.promise(table.fields);
                    });
                } else {
                    //$scope.fields = table.fields;
                    console.log("scope tables get fields", $scope.tables, $scope.selectedTable.fields);
   
                    return qvangular.promise(table.fields);
                }

            }

            $scope.showFieldDataLoader = function () {
                return $scope.loadInProgress || $scope.loadingPreview > 0;
            };

            $scope.isTableSelected = function (table) {
                return (table === $scope.selectedTable);
            };


            //Internal Functions


            // This accomodates the recently changed QlikView API contract
            // without having to make changes to the contract for the connect dialog.
            ///// see C:\Users\micha\AppData\Local\Programs\Common Files\Qlik\Custom Data\QvOdbcConnectorPackage\web\selectdialog.js ///

            function indentLine(level) {
                var indent = "";
                for (var i = 0; i < level; i++) {
                    indent += "\t";
                }
                return indent;
            }

            //build query from selection
            function GetScriptFromQuery(tables, limit) {
                console.log("Build script from GUI");

                var selectedTables = tables.filter(table => table.checked);
                console.log("found selected tables", selectedTables);

                var script = "";

                //loop tables
                selectedTables.forEach(function (table) {
                    var query = "";
                    var queryFields = "";

                    query = "[" + table.name + "]:\r\n";
                    query += "SELECT\r\n";

                    //find selected fields
                    var seletedFields = table.fields.filter(field => field.checked);
                    console.log("selected fields", seletedFields);

                    //loop selected fields
                    seletedFields.forEach(function (field) {
                        queryFields += indentLine(1) + "[" + field.id + "],\r\n";
                    });

                    //add query fields
                    query += queryFields.slice(0, -3) + "\r\n";

                    //add table reference
                    query += indentLine(0) + "FROM [" + table.name + "]";

                    //add limit for preview
                    if (!!limit && Number.isInteger(limit) && limit > 0) {
                        query += indentLine(1) + "Limit " + limit;
                    }

                    script += GetCurrentScript(query, table.name);
                    
                });

                
                   
                return script;

            }

            //Query wrapper
            function GetCurrentScript(serviceNowQueryInput, tableName) {
                console.log("Generate Script", serviceNowQueryInput, tableName);
                var script = "";
                var serviceNowQuery = "";

                //add indent to lines of query
                serviceNowQueryInput.split(/\r?\n/).forEach(function (line) {
                    serviceNowQuery += indentLine(0) + line + "\r\n";
                });

                //script += indentLine(0) + "SELECT\r\n";
                //script += indentLine(1) + "QUERY\r\n";
                script += indentLine(0) + serviceNowQuery.slice(0, -2) + "\r\n";
                script += ";";


                console.log("Generated Script", serviceNowQueryInput, script);

                script += "\r\n\r\n";
                return script;
            }
            

              ///// Event handlers //////

            //tables
            $scope.onTableShowClick = function ($event) {
                console.log("onTableShowClick", $event);
                if (!$scope.editLockdown) {
                    var table = $($event.target).scope().table;
                    $scope.onTableShow(table);
                }
            };

            $scope.onTableShow = function (table) {
                if (table && table !== $scope.selectedTable) {
                    $scope.selectedTable = table;

                    getFields(table.name);
                }
            };

            $scope.onTableCheckboxClick = function ($event) {
                console.log("onTableCheckboxClick", $event);
                if (!$scope.editLockdown) {
                    var targetScope = $($event.target).scope();
                    var table = targetScope.table;
                    table.proptest = 2;

                    getFields(table.name).then(function (fields) {
                        //udate the table status
                        if (!table.checked) {
                            table.checked = true;
                        } else {
                            table.checked = !table.checked;
                        }

                        //set same status as the table to all fields
                        fields.forEach(function (field) {
                            field.checked = table.checked;   
                        });
                        //update date number over selected fields (could be move to a watch)
                        table.numberOfSelectedFields = fields.length;
                    });
                }
                console.log("scope tables on check table", $scope.tables);
            };

            //fields
            $scope.handleFieldClick = function ($event) {

                //console.log("event", $event);

                var targetScope = $($event.target).scope();
                var field = targetScope.field;

                console.log("handleFieldClick", field);

                if (!field.checked) {
                    field.checked = true;
                } else {
                    field.checked = !field.checked;
                }

                var checkedFields = $scope.selectedTable.fields.filter(field => field.checked).length;

                $scope.selectedTable.numberOfSelectedFields = checkedFields;

                //check the table has checked fields (not optimized for readability)
                if (checkedFields > 0) {
                    $scope.selectedTable.checked = true;                    
                } else {
                    $scope.selectedTable.checked = false;
                }
            };

            $scope.onDataPreviewClick = function () {
                $scope.dataview = true;
                $scope.helpview = false;
                $scope.metadataview = false;

                //if (!$scope.preview || $scope.preview.length === 0) {
                //    loadDataPreview();
                //}
            };

            $scope.onContentConfigClick = function () {
                $scope.dataview = false;
                $scope.helpview = false;
                $scope.metadataview = true;
            };


            $scope.onHelpClick = function () {
                $scope.dataview = false;
                $scope.helpview = true;
                $scope.metadataview = false;
            };


            $scope.isAddScriptButtonEnabled = function () {

                //checks for an error state in the connector
                if (!!$scope.selectDialogState && $scope.selectDialogState != "ERROR") {
                     //NOT GUI and check a query is pressent
                    if (!$scope.showSelectionGUI && !!$scope.serviceNowQuery && $scope.serviceNowQuery.length > 0) {
                        //check project is defined
                        if (!!$scope.currentProject.id && $scope.currentProject.id.length > 0) {
                            return true;
                        }
                    }
                    //GUI and selected are pressent 
                    if ($scope.showSelectionGUI && $scope.tables.filter(table => table.checked).length > 0) {
                        return true;
                    }
                }
                return false;
            };


            //SEE Rest connector C:\Users\micha\AppData\Local\Programs\Common Files\Qlik\Custom Data\QvRestConnector\web\selectdialog.js
            $scope.onOK = function () {
                $scope.isLoading = true;

                if (!$scope.showSelectionGUI) {
                    console.log("Script from direct query");
                    var script = GetCurrentScript($scope.serviceNowQuery, "ServiceNow");
                } else {
                    console.log("Script from GUI selection");
                    var script = GetScriptFromQuery($scope.tables, null);
                }


                $scope.scriptEditorProxy.insertCode(script, $scope.dataconnectorInstance);
                $scope.destroyComponent();
            };

            $scope.onCancel = function () {
                $scope.cancelIsInProgress = true;
                $scope.destroyComponent();

            };

            //init model

            init();

        }]
    };
});



